


#import <eoaccess/eoaccess.h>
#import "NSString+MiscRegex.h"
#import <foundation/NSException.h>
#import <foundation/NSAutoreleasePool.h>
#import <misckit/regexpr.h>
#import <stdio.h>


const char *strFromRange(NSRange range)
{
	NSString *theString;

	if (range.location == NSNotFound)
		theString = [NSString stringWithFormat:@"{location = NSNotFound; length = %d}",
							range.length];
	else
		theString = [NSString stringWithFormat:@"{location = %d; length = %d}",
							range.location, range.length];
	
	return [theString cString];
}

void main(int argc, char *argv[])
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	NSString		*testsFile  = (argv[1])? [NSString stringWithCString:argv[1]] : 
	                                         @"MiscRegexTest.tests";
	NSString		*fileString = [[NSString alloc] initWithContentsOfFile:testsFile];
	NSArray			*strings    = [fileString componentsSeparatedByString:@"\n"];
	NSEnumerator	*stringEnum = [strings objectEnumerator];
	NSString		*currLine;
	int num, i;


	re_set_syntax(RE_NO_BK_VBAR | RE_NO_BK_PARENS | RE_CONTEXT_INDEP_OPS);

	while ((currLine = [stringEnum nextObject]) && [currLine length])
	{
		NSArray		*tokens       = [currLine componentsSeparatedByString:@"\t"];
		NSString	*currRegex    = [tokens objectAtIndex:0];
		NSString	*currString   = [tokens objectAtIndex:1];
		NSString	*expect       = [tokens objectAtIndex:2];
		NSString	*currReplace  = [tokens objectAtIndex:3];
		NSString	*afterReplace = [tokens objectAtIndex:4];
		NSString	*afterString;
		BOOL		isValid;
		NSRange		foundRange;

		if ([currRegex isEqual:@"-"]) continue;

	NS_DURING
			
		printf("String is: \"%s\"\n", [currString cString]);
		printf("Regex is:  \"%s\"\n\n", [currRegex cString]);
		

		isValid = [currRegex isValidRegex];
		printf("Valid regex?         %s\n", isValid? "YES":"NO");
		if (!isValid && ![expect isEqual:@"c"])
			printf("ERROR**: regex '%s' supposed to be valid\n",[currRegex cString]);
		if (isValid && [expect isEqual:@"c"])
			printf("ERROR**: regex should have been illegal\n");


		foundRange = [currString rangeOfRegex:currRegex options:MiscFasterSearch];
		printf("Range of first occurrence: %s\n", strFromRange(foundRange));
		
		if (foundRange.location == NSNotFound && [expect isEqual:@"y"])
			printf("ERROR**: regex not found but should have been\n");
		if (foundRange.location != NSNotFound && [expect isEqual:@"n"])
			printf("ERROR**: regex found but should not have been\n");
		
		
		printf("Num               : %d\n", num = [currString numOfRegex:currRegex]);
		printf("Num nocase        : %d\n",
				[currString numOfRegex:currRegex options:NSCaseInsensitiveSearch]);
				
		for (i=-1; i <= num; i++)
		{
			NSRange findRange = [currString rangeOfRegex:currRegex occurrenceNum:i];
			printf("          Occurrence %2d: %s\n", i, strFromRange(findRange));
		}

		for (i=-1; i <=num; i++)
		{
			NSRange findRange = [currString rangeOfRegex:currRegex
			                      options:NSBackwardsSearch occurrenceNum:i];
			printf("Backwards Occurrence %2d: %s\n", i, strFromRange(findRange));
		}

		for (i=0; i <=1; i++)
		{
			NSRange findRange = [currString rangeOfRegex:currRegex
			                      options:NSAnchoredSearch occurrenceNum:i];
			printf("Anchored  Occurrence %2d: %s\n", i, strFromRange(findRange));
		}

		for (i=0; i <=1; i++)
		{
			NSRange findRange = [currString rangeOfRegex:currRegex
			                      options:NSAnchoredSearch|NSBackwardsSearch
								  occurrenceNum:i];
			printf("Back,Anch Occurrence %2d: %s\n", i, strFromRange(findRange));
		}

		if (![currReplace isEqual:@"-"] && foundRange.location != NSNotFound)
		{
			NSString *substring = [currString substringFromRange:foundRange];
			afterString = [substring stringByReplacingRegex:currRegex withString:currReplace
								options:MiscUseMatchSubstitutions];
			if (![afterString isEqual:afterReplace])
				printf("ERROR**: replace didn't work: was '%s', should be '%s'\n",
							[afterString cString], [afterReplace cString]);
		}

		
		if (![currReplace isEqual:@"-"] && ![currReplace isEqual:@"&"])
		{
			afterString = [currString stringByReplacingEveryOccurrenceOfRegex:currRegex
						withString:currReplace options:MiscUseMatchSubstitutions];
			printf("After replace all with '%s': '%s'\n",
						[currReplace cString], [afterString cString]);
		}
		afterString = [currString stringByReplacingEveryOccurrenceOfRegex:currRegex
						withString:@"[&]" options:MiscUseMatchSubstitutions];
		printf("After replace all with '[&]': '%s'\n", [afterString cString]);
		
		printf("Matches for: %s\n",
			[[[currString stringsMatchedByRegex:currRegex] description] cString]);
		
		printf("Components separated by: %s\n",
			[[[currString componentsSeparatedByRegex:currRegex] description] cString]);
		
		for (i=-1; i <= num; i++)
		{
			NSArray			*keys = [NSArray arrayWithObjects:@"before", @"middle",
										@"after", @"substrings",nil];
			NSDictionary	*dict =	[currString grep:currRegex forKeys:keys occurrenceNum:i];

			printf("Occurrence %d\n",i);
			printf("   Before: '%s'\n",[[dict objectForKey:@"before"] cString]);
			printf("   Middle: '%s'\n",[[dict objectForKey:@"middle"] cString]);
			printf("   After:  '%s'\n",[[dict objectForKey:@"after"] cString]);
			printf("   Subs:  %s\n",[[[dict objectForKey:@"substrings"] description] cString]);
		}
		
		printf("----------------------------------------------------------\n");


	NS_HANDLER
			
		printf("%s: %s\n",[[exception exceptionName] cString],
		                  [[exception exceptionReason] cString]);
		printf("----------------------------------------------------------\n");

	NS_ENDHANDLER
	}
	

	[pool release];
}
