
#import "NSScanner+MiscRegex.h"
#import "NSString+MiscRegex.h"


@interface NSScanner (Misc_NeXT_Private)
/*
 * This skips past any "skip" characters, updating the scanLocation. It
 * returns an NSCharacterSet, presumably the charactersToSkip set.
 */
- (NSCharacterSet *)skipCharacters;
@end


@implementation NSScanner (MiscRegex)

- (BOOL)scanRegex:(NSString *)regex intoString:(NSString **)value
{
	NSString	*theString = [self string];
	NSRange		scanRange;
	NSRange		foundRange;
	int			options = NSAnchoredSearch;
	
	if (![self caseSensitive]) options |= NSCaseInsensitiveSearch;
	
	[self skipCharacters];

	scanRange.location = [self scanLocation];
	scanRange.length   = [theString length] - scanRange.location;

	foundRange = [theString rangeOfRegex:regex options:options range:scanRange];
	
	if (foundRange.location == NSNotFound)
	{
		return NO;
	}
	else
	{
		if (value) *value = [theString substringFromRange:foundRange];
		[self setScanLocation:NSMaxRange(foundRange)];
		return YES;
	}
}

- (BOOL)scanUpToRegex:(NSString *)regex intoString:(NSString **)value
{
	NSString	*theString = [self string];
	NSRange		scanRange;
	NSRange		foundRange;
	NSRange		betweenRange;
	int			options = 0;
	
	if ([self isAtEnd]) return NO;

	if (![self caseSensitive]) options |= NSCaseInsensitiveSearch;

	[self skipCharacters];

	scanRange.location = [self scanLocation];
	scanRange.length   = [theString length] - scanRange.location;
	
	foundRange = [theString rangeOfRegex:regex options:options range:scanRange];

	if (foundRange.location == NSNotFound)
	{
		[self setScanLocation:[theString length]];
		if (value) *value = [theString substringFromRange:scanRange];
		return YES;
	}
	else
	{
		betweenRange.location = scanRange.location;
		betweenRange.length = foundRange.location - scanRange.location;
	
		if (betweenRange.length == 0)
		{
			return NO;
		}
		else
		{
			[self setScanLocation:foundRange.location];
			if (value) *value = [theString substringFromRange:betweenRange];
			return YES;
		}
	}
}
	

@end

