//
//	NSString+MiscRegex.h -- add regular expression support to NSString
//		Written by Carl Lindberg Copyright (c) 1996 by Carl Lindberg.
//				  Version 0.9   All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import <foundation/NSString.h>

/*
 * Additional flags you can pass with the options parameter.
 * 
 * MiscFasterSearch will use the fastmap feature of the regexpr package.
 * This will result in faster searches on longer strings, but the overhead
 * may not be worth it with short strings.
 * 
 * MiscUseMatchSubstitutions will turn on special processing on the
 * 'replace string' used in -stringByReplacingEveryOccurrenceOfRegex...
 * stringByReplacingRegex... methods. A string of the form '$<digit>' will
 * be replaced with the text matched by the <digit>'th parenthesized
 * subexpression.  For example, for the regex "aa(b+)", "$1" would be
 * replaced with the string of b's matched by the "b+" part of the regex.
 * "$0" and "&" are special tokens that will be replaced with the entire
 * matched text. Use "$$" and "$&" if you want a literal '$' or '&'.
 */
enum
{
    MiscFasterSearch = 4096,
    MiscUseMatchSubstitutions = 8192
};


extern NSString *MiscRegexException;


/* Keys used for the grep: methods */

extern NSString *MiscBeforeKey;
extern NSString *MiscMiddleKey;
extern NSString *MiscAfterKey;
extern NSString *MiscBeforeRangeKey;
extern NSString *MiscMiddleRangeKey;
extern NSString *MiscAfterRangeKey;
extern NSString *MiscSubstringsKey;
extern NSString *MiscSubrangesKey;

@class NSDictionary, NSArray;


@interface NSString (MiscRegex)

- (unsigned)numOfRegex:(NSString *)regex options:(unsigned)mask;
- (unsigned)numOfRegex:(NSString *)regex options:(unsigned)mask range:(NSRange)range;

- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask occurrenceNum:(int)n;
- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask occurrenceNum:(int)n
			range:(NSRange)range;

- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys
			options:(unsigned)mask occurrenceNum:(int)n;
- grep:(NSString *)regex forKey:(NSString *)key options:(unsigned)mask occurrenceNum:(int)n;

- (id)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex
			withString:(NSString *)aString options:(unsigned)mask;
- (id)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex
			withString:(NSString *)aString options:(unsigned)mask range:(NSRange)range;

- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString
			options:(unsigned)mask occurrenceNum:(int)n;
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
			options:(unsigned)mask occurrenceNum:(int)n range:(NSRange)range;

- (NSArray*)stringsMatchedByRegex:     (NSString *)regex options:(unsigned)mask;
- (NSArray*)rangesMatchedByRegex:      (NSString *)regex options:(unsigned)mask;
- (NSArray*)componentsSeparatedByRegex:(NSString *)regex options:(unsigned)mask;

- (BOOL)isValidRegex;


#ifndef _HEADER_VIEWER_  /* convenience methods */

- (unsigned)numOfRegex:(NSString *)regex;
- (unsigned)numOfRegex:(NSString *)regex range:(NSRange)range;

- (NSRange)rangeOfRegex:(NSString *)regex;
- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask;
- (NSRange)rangeOfRegex:(NSString *)regex occurrenceNum:(int)n;
- (NSRange)rangeOfRegex:(NSString *)regex range:(NSRange)range;
- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask range:(NSRange)range;
- (NSRange)rangeOfRegex:(NSString *)regex occurrenceNum:(int)n range:(NSRange)range;

- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys;
- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys options:(unsigned)mask;
- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys occurrenceNum:(int)n;
- grep:(NSString *)regex forKey:(NSString *)keys;
- grep:(NSString *)regex forKey:(NSString *)keys options:(unsigned)mask;
- grep:(NSString *)regex forKey:(NSString *)keys occurrenceNum:(int)n;

- (id)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex withString:(NSString *)aString;
- (id)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex withString:(NSString *)aString 
				range:(NSRange)range;

- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString;
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
				options:(unsigned)mask;
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
				occurrenceNum:(int)n;
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
				range:(NSRange)range;
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
				options:(unsigned)mask range:(NSRange)range;
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
				occurrenceNum:(int)n range:(NSRange)range;

- (NSArray*)stringsMatchedByRegex:(NSString *)regex;
- (NSArray*)rangesMatchedByRegex:(NSString *)regex;
- (NSArray*)componentsSeparatedByRegex:(NSString *)regex;

#endif

@end

